/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "CharacterData.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCharacterData.h"
#include "gdom/GdomEventTargetNode.h"
#include "gdom/PrivateGdomCharacterData.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomEventTargetNode.h"
#include "gdom/PrivateIntGdomCharacterData.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMCharacterData::GDOMCharacterData(PassRefPtr<WebCore::CharacterData> impl)
    : GDOMEventTargetNode(impl)
{
}

gchar *
gdom_character_data_substring_data (GdomCharacterData *thiz, gulong offset, gulong length)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CharacterData * item = (gdom_character_data_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    gchar * res = _G_GET_UTF8(item->substringData(offset, length, ec));
    return res;

}

void
gdom_character_data_append_data (GdomCharacterData *thiz, gchar * data)
{
    g_return_if_fail (thiz);
    WebCore::CharacterData * item = (gdom_character_data_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (data);
    WebCore::String _g_data = WebCore::String::fromUTF8(data);
    WebCore::ExceptionCode ec = 0;
    item->appendData(_g_data, ec);

}

void
gdom_character_data_insert_data (GdomCharacterData *thiz, gulong offset, gchar * data)
{
    g_return_if_fail (thiz);
    WebCore::CharacterData * item = (gdom_character_data_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (data);
    WebCore::String _g_data = WebCore::String::fromUTF8(data);
    WebCore::ExceptionCode ec = 0;
    item->insertData(offset, _g_data, ec);

}

void
gdom_character_data_delete_data (GdomCharacterData *thiz, gulong offset, gulong length)
{
    g_return_if_fail (thiz);
    WebCore::CharacterData * item = (gdom_character_data_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->deleteData(offset, length, ec);

}

void
gdom_character_data_replace_data (GdomCharacterData *thiz, gulong offset, gulong length, gchar * data)
{
    g_return_if_fail (thiz);
    WebCore::CharacterData * item = (gdom_character_data_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (data);
    WebCore::String _g_data = WebCore::String::fromUTF8(data);
    WebCore::ExceptionCode ec = 0;
    item->replaceData(offset, length, _g_data, ec);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "CharacterData.h"
#include <gdom/GdomCharacterData.h>
#include <gdom/PrivateGdomCharacterData.h>
#include <gdom/PrivateIntGdomCharacterData.h>
#include <gdom/GdomCharacterDataprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomCharacterData, gdom_character_data, GDOM_TYPE_EVENT_TARGET_NODE);

/*
struct _GdomCharacterDataPrivate {
    class WebCore::CharacterData *item;
};

#define GDOM_CHARACTER_DATA_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_CHARACTER_DATA, GdomCharacterDataPrivate))
*/
G_END_DECLS
WebCore::CharacterData *
gdom_character_data_get_private(GdomCharacterData *request)
{
    GDOMCharacterData *res;
    res = static_cast<GDOMCharacterData*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::CharacterData*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_DATA,
    PROP_LENGTH,
};


static void gdom_character_data_finalize(GObject* object)
{
    /*GdomCharacterData* request = GDOM_CHARACTER_DATA(object);*/
    /*GdomCharacterDataPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_EVENT_TARGET_NODE));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_character_data_parent_class)->finalize(object);
}

static void gdom_character_data__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomCharacterData *self = GDOM_CHARACTER_DATA(object);
    WebCore::CharacterData * gself = (gdom_character_data_get_private(self));
    switch (prop_id) {
     case PROP_DATA:
    {
         WebCore::ExceptionCode ec = 0;
         gself->setData(WebCore::String::fromUTF8(g_value_get_string (value)) , ec );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_character_data__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomCharacterData *self = GDOM_CHARACTER_DATA(object);
    WebCore::CharacterData * gself = (gdom_character_data_get_private(self));
    switch (prop_id) {
    case PROP_DATA:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->data()));
         break;
    }
    case PROP_LENGTH:
    {
         g_value_set_ulong (value, gself->length());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_character_data_class_init(GdomCharacterDataClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_character_data_finalize;
    g_klass->set_property = gdom_character_data__set_property_fn;
    g_klass->get_property = gdom_character_data__get_property_fn;

    g_object_class_install_property (g_klass, PROP_DATA,
            g_param_spec_string ("data", /* name */
                "character_data_data", /* short description */
                "read-write  gchar * CharacterData.data", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_ulong ("length", /* name */
                "character_data_length", /* short description */
                "read-only  gulong CharacterData.length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomCharacterDataPrivate));*/
}

static void gdom_character_data_init(GdomCharacterData* request)
{
    /*GdomCharacterDataPrivate* priv = GDOM_CHARACTER_DATA_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

